@@ -3,13 +3,13 @@ from .lstcontainers import (
     DL1ParametersContainer,
     DispContainer,
 )
+from .event_selection import EventSelector, DL3FixedCuts, DataBinning
 from .io import (
     get_dataset_keys,
     auto_merge_h5files,
     smart_merge_h5files,
     write_simtel_energy_histogram,
     write_mcheader,
-    write_array_info,
     write_dl2_dataframe,
     global_metadata,
     add_global_metadata,
@@ -17,23 +17,26 @@ from .io import (
     write_subarray_tables,
     read_simu_info_merged_hdf5,
     write_calibration_data,
+    read_mc_dl2_to_QTable,
+    read_data_dl2_to_QTable,
     HDF5_ZSTD_FILTERS,
 )
 
 standard_config = get_standard_config()
 
 __all__ = [
-    'get_standard_config',
     'replace_config',
     'read_configuration_file',
     'DL1ParametersContainer',
     'DispContainer',
+    'EventSelector',
+    'DL3FixedCuts',
+    'DataBinning',
     'get_dataset_keys',
     'auto_merge_h5files',
     'smart_merge_h5files',
     'write_simtel_energy_histogram',
     'write_mcheader',
-    'write_array_info',
     'write_dl2_dataframe',
     'global_metadata',
     'add_global_metadata',
@@ -42,4 +45,7 @@ __all__ = [
     'read_simu_info_merged_hdf5',
     'write_calibration_data',
     'HDF5_ZSTD_FILTERS',
+    'read_mc_dl2_to_QTable',
+    'read_data_dl2_to_QTable',
+
 ]
