@@ -32,6 +32,7 @@ from lstchain.io.config import read_configuration_file, replace_config
 from lstchain.io.io import dl1_params_lstcam_key, dl1_images_lstcam_key
 from lstchain.io.lstcontainers import DL1ParametersContainer
 from lstchain.reco.disp import disp
+from lstchain.image.modifier import smear_light_in_pixels, add_noise_in_pixels
 
 log = logging.getLogger(__name__)
 
@@ -67,6 +68,12 @@ parser.add_argument('--pedestal-cleaning', action='store',
                     help='Boolean. True to use pedestal cleaning',
                     default=False)
 
+parser.add_argument('--dynamic-cleaning', action='store',
+                    type=lambda x: bool(strtobool(x)),
+                    dest='dynamic_cleaning',
+                    help='Boolean. True to use dynamic cleaning',
+                    default=False)
+
 args = parser.parse_args()
 
 
@@ -82,6 +89,24 @@ def main():
     else:
         config = std_config
 
+    increase_nsb = False
+    increase_psf = False
+    if "image_modifier" in config:
+        imconfig = config["image_modifier"]
+        increase_nsb = imconfig["increase_nsb"]
+        increase_psf = imconfig["increase_psf"]
+        if increase_nsb or increase_psf:
+            log.info(f"image_modifier configuration: {imconfig}")
+        extra_noise_in_dim_pixels = imconfig["extra_noise_in_dim_pixels"]
+        extra_bias_in_dim_pixels = imconfig["extra_bias_in_dim_pixels"]
+        transition_charge = imconfig["transition_charge"]
+        extra_noise_in_bright_pixels = imconfig["extra_noise_in_bright_pixels"]
+        smeared_light_fraction = imconfig["smeared_light_fraction"]
+        if (increase_nsb or increase_psf) and args.noimage is False:
+            log.info("NOTE: Using the image_modifier options means images will "
+                     "not be saved.")
+            args.noimage = True
+
     if args.pedestal_cleaning:
         log.info("Pedestal cleaning")
         clean_method_name = 'tailcuts_clean_with_pedestal_threshold'
@@ -100,6 +125,14 @@ def main():
         picture_th, boundary_th, isolated_pixels, min_n_neighbors = cleaning_params
         log.info(f"Tailcut config used: {config['tailcut']}")
 
+    if args.dynamic_cleaning:
+        THRESHOLD_DYNAMIC_CLEANING = config['dynamic_cleaning']['threshold']
+        FRACTION_CLEANING_SIZE = config['dynamic_cleaning']['fraction_cleaning_intensity']
+        log.info("Using dynamic cleaning for events with average size of the "
+            f"3 most brighest pixels > {config['dynamic_cleaning']['threshold']} p.e")
+        log.info("Remove from image pixels which have charge below "
+                 f"= {config['dynamic_cleaning']['fraction_cleaning_intensity']} * average size")
+    
     use_only_main_island = True
     if "use_only_main_island" in config[clean_method_name]:
         use_only_main_island = config[clean_method_name]["use_only_main_island"]
@@ -153,8 +186,13 @@ def main():
         if set(dl1_params_input).intersection(disp_params):
             parameters_to_update.extend(disp_params)
 
+        if increase_nsb:
+            rng = np.random.default_rng(
+                    input.root.dl1.event.subarray.trigger.col('obs_id')[0])
+
         with tables.open_file(args.output_file, mode='a') as output:
             params = output.root[dl1_params_lstcam_key].read()
+
             for ii, row in enumerate(image_table):
 
                 dl1_container.reset()
@@ -162,12 +200,29 @@ def main():
                 image = row['image']
                 peak_time = row['peak_time']
 
+                if increase_nsb:
+                    # Add noise in pixels, to adjust MC to data noise levels.
+                    # TO BE DONE: in case of "pedestal cleaning" (not used now
+                    # in MC) we should recalculate picture_th above!
+                    image = add_noise_in_pixels(rng, image,
+                                                extra_noise_in_dim_pixels,
+                                                extra_bias_in_dim_pixels,
+                                                transition_charge,
+                                                extra_noise_in_bright_pixels)
+                if increase_psf:
+                    image = smear_light_in_pixels(image,
+                                                  camera_geom,
+                                                  smeared_light_fraction)
+            
+                
+
                 signal_pixels = tailcuts_clean(camera_geom,
                                                image,
                                                picture_th,
                                                boundary_th,
                                                isolated_pixels,
                                                min_n_neighbors)
+    
 
                 n_pixels = np.count_nonzero(signal_pixels)
                 if n_pixels > 0:
@@ -191,6 +246,17 @@ def main():
                                                              1, delta_time)
                         signal_pixels = new_mask
 
+                    if args.dynamic_cleaning:
+                        max_3_value_index = np.argsort(image)[-3:]
+                        mean_3_max_signal = np.mean(image[max_3_value_index])
+                        if mean_3_max_signal > THRESHOLD_DYNAMIC_CLEANING:
+                            cleaned_img = image.copy()
+                            cleaned_img[~signal_pixels] = 0
+                            dynamic_threshold = FRACTION_CLEANING_SIZE*mean_3_max_signal
+                            mask_dynamic_cleaning = (cleaned_img > 0) & (cleaned_img < dynamic_threshold)
+                            new_mask_after_dynamic_cleaning = ~np.logical_or(~signal_pixels, mask_dynamic_cleaning)
+                            signal_pixels = new_mask_after_dynamic_cleaning
+
                     # count the surviving pixels
                     n_pixels = np.count_nonzero(signal_pixels)
 
