@@ -76,9 +76,7 @@ def main():
 
     if args.noimage:
         keys = get_dataset_keys(file_list[0])
-        for k in keys:
-            if 'image' in k:
-                keys.remove(k)
+        keys = {k for k in keys if 'image' not in k}
     else:
         keys = None
 
